; --------------------------------------------------------------------------------
; DETERMINACAO DA FAMILIA DO MICROCONTROLADO CORRENTE                   06/08/2011
;
; Esta macro gera o simbolbo __FAMILIA identificando o tipo de microcontrolador
; que esta sendo usado, permitindo que outraas macros gerem codigos apropriados
; para cada um.
;
; --------------------------------------------------------------------------------
FAMILIAS  MACRO

          VARIABLE     __FAMILIA
__FAMILIA SET          0

          IFDEF  __12F675
__FAMILIA SET    12
          ENDIF

          IFDEF  __16F688
__FAMILIA SET    16
          ENDIF

          IFDEF  __16F628A
__FAMILIA SET    16
          ENDIF

          IFDEF  __16F73
__FAMILIA SET    16
          ENDIF

          IFDEF  __16F873A
__FAMILIA SET    16
          ENDIF

          IFDEF  __16F877A
__FAMILIA SET    16
          ENDIF

          IFDEF  __18F2550
__FAMILIA SET    18
          ENDIF

          IFDEF  __18F4520
__FAMILIA SET    18
          ENDIF

          IFDEF  __18F4620
__FAMILIA SET    18
          ENDIF

          IFDEF  __18F4550
__FAMILIA SET    18
          ENDIF

          ENDM



; ----------------------------------------------------------------------------------
; HEX2ASCII                                                     Spinelli 26/03/2009
; Monta a tabela ASCII de valores de 0 a F.
; Esta macro e usada pela subrotina HEXAG.
; ----------------------------------------------------------------------------------
HEX2ASCII    MACRO

      IF __FAMILIA==18
HEX_ASCII    DB       "0123456789ABCDEF",0,0      ; suporte a familia 18F

; VERSAO "TABLE READ" PARA FAMILIA 18F
;HEX_ASCII    addwf    PCL,f
;             DT       "0123456789ABCDEF",0,0      ; suporte a familia 16F
      ENDIF

      IF __FAMILIA<=16
HEX_ASCII    addwf    PCL,f
             DT       "0123456789ABCDEF",0,0      ; suporte a familia 16F
    ENDIF
             ENDM





; --------------------------------------------------------------------------------------------------------------
; USART_CONFIG - CONFIGURACAO DA USART                                                              24/08/2011
;
;    Esta macro configura a USART nos modos sincrono ou assincrono considerando o valor do
;    baud rate desejado para a geracao do Baud Rate Generator (BRG). Permite tambem que se
;    faca a opcao por gerar a interrupcao de Rx.
;    Esta macro habilita a porta serial, o receiver (Rx) e o transmitter (tx).
;    A configuracao de I/O para Tx e Rx sera feita somente para a familia 18F.
;
;
;    USART_CONFIG MACRO  bauds,clk, mode,interrupt
; 
;    bauds.......................... Baud Rate (BPS) desejado (4800, 9600, 19200, etc)
;    clk............................ frequenciao do clock (Fosc)
;    mode........................... "A" - modo assincrono     
;                                    "S" - modo sincrono
;    interrupt ..................... interrupcao de Rx
;                                    "Y" - com interrupcao de Rx
;                                    "N" - sem interrupcao de Rx
;    intertype ..................... "L" - interrupcao ed Rx com baixa prioridade
;                                    "H" - interrupcao ed Rx com alta prioridade
;                                    "C" - interrupcao de Rx no modo normal (compativel)
;
;   REQUISITOS
;   ----------
;   1 - ESTA MACRO REQUER O USO DA MACRO "FAMILIAS" PARA GERAR O CODIGO ADEQUADO PARA O MICROCONTROLADOR
;       ESTA SENDO USADO.
;
;   2 - ESTA MACRO REQUER A DEFINICAO DO SIMBOLO "CLOCK" COM O VALOR DA FREQUENCIA DO OSCILADOR (Fosc).
;
; --------------------------------------------------------------------------------------------------------------
USART_CONFIG  MACRO    bauds,clk,mode,interrupt,intertype
  ifndef usart_id
            variable usart_id 
usart_id    set   1
txon        equ   1                     ; USART_set txon ........ habilita transmissao
txoff       equ   2                     ; USART_set txoff ....... desabilita transmissao
rxon        equ   3                     ; USART_set rxon ........ habilita recepcao
rxoff       equ   4                     ; USART_set rxoff ....... desabilita recepcao
usarton     equ   5                     ; USART_set usarton ..... habilita a porta serial
usartoff    equ   6                     ; USART_set usartoff .... desabilita a porta serial
  endif

  IF __FAMILIA == 18

         IF (mode == "A") && (bauds < .9600)
             bcf     TXSTA,SYNC                       ; modo assincrono
             bcf     BAUDCON,BRG16                    ; diz que SPBRG tem 8 bits
             bcf     TXSTA,BRGH                       ; estabelece baixa velocidade
             local  brg = clk / ( .64 * (bauds + .1))
         ENDIF

         IF (mode == "A") && (bauds >= .9600) 
             bcf     TXSTA,SYNC                       ; modo assincrono
             bsf     BAUDCON,BRG16                    ; forca 16 bits em SPBRG
             bsf     TXSTA,BRGH                       ; estabelece alta velocidade
             local  brg = clk / ( .4 * (bauds + .1))
         ENDIF

         IF (mode == "S")
             bsf     TXSTA,SYNC                       ; modo sincrono
             bsf     BAUDCON,BRG16                    ; forca 16 bits em SPBRG
             local  brg = clk / ( .4 * (bauds + .1))
         ENDIF

             movlw   0x80                             ; configuracao de I/O 
             iorwf   TRISC,F                          ;

             movlw   HIGH(brg)                        ; baud rate high

             movwf   SPBRGH                           ; estabelece o baud rate (high)
             movlw   LOW(brg)                         ; baud rate low

             movwf   SPBRG                            ; estabelece o baud rate (low)

	         bsf     RCSTA,SPEN                       ; habilita porta serial
             bsf     RCSTA,CREN                       ; habilita o receiver
             bsf     TXSTA,TXEN                       ; habilita o transmitter

         IF (interrupt == "Y") && (intertype == "H")
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     RCON,IPEN                        ; habilita prioridade de interrupcao
             bsf     IPR1,RCIP                        ; habilita interrupcao de alta prioridade em RX
             bsf     INTCON,GIEH                      ; habilita interrupcoes de alta prioridade
         ENDIF

         IF (interrupt == "Y") && (intertype == "L")
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     RCON,IPEN                        ; habilita prioridade de interrupcao
             bcf     IPR1,RCIP                        ; habilita interrupcao de baixa prioridade em RX
             bsf     INTCON,GIEL                      ; habilita interrupcoes de baixa prioridade
         ENDIF

         IF (interrupt == "Y") && (intertype == "C")
             bsf     PIE1,RCIE                        ; habilita interrupcoes em RX
             bsf     INTCON,PEIE                      ; habilita interrupcoes de perifericos
             bsf     INTCON,GIE                       ; habilita interrupcoes globalmente
         ENDIF

  ENDIF







  IF __FAMILIA == 16

             bsf     STATUS,RP0
         IF (mode == "A") && (bauds < .9600)
             bcf     TXSTA,SYNC                  ; modo assincrono
             bcf     TXSTA,BRGH                  ; estabelece baixa velocidade
             local   brg = clk / ( .64 * (bauds + .1))
         ENDIF

         IF (mode == "A") && (bauds >= .9600) 
             bcf     TXSTA,SYNC                  ; modo assincrono
             bsf     TXSTA,BRGH                  ; estabelece alta velocidade
             local   brg = clk / ( .16 * (bauds + .1))
         ENDIF

         IF (mode == "S")
             bsf     TXSTA,SYNC                  ; modo sincrono
             bcf     TXSTA,BRGH                  ; estabelece baixa velocidade
             local   brg = clk / ( .4 * (bauds + .1))
         ENDIF

             movlw   brg                         ; valor para o Baud Rate de recepcao
             movwf   SPBRG                       ; especifica Baud Rate

         IF interrupt == "Y"
             bsf     PIE1,RCIE                   ; habilita interrupcao de Rx
         ENDIF 

             bsf     TXSTA,TXEN                  ; habilita a transmissao
             bcf     STATUS,RP0                  ; banco 0
             bsf     RCSTA,CREN                  ; habilita a recepcao
             bsf     RCSTA,SPEN                  ; habilita porta serial

         IF interrupt == "Y"
             bsf     INTCON,PEIE                 ; interrupcoes de perifericos
             bsf     INTCON,GIE                  ; interrupcoes globais
         ENDIF 

             movf    RCREG,w                     ;
             movf    RCREG,w                     ; limpa o buffer de leitura
             movf    RCREG,w                     ;      

  ENDIF


              ENDM






; --------------------------------------------------------------------------------------------------------------
; MACRO: USART_RX                                                                         Spinelli - 15/02/2011
;
; Le a porta RX da USART e retorna o caracter lido no registrador W
;
; --------------------------------------------------------------------------------------------------------------
USART_RX     MACRO
_numq        set     _numq + 1
RCV#v(_numq)
             banksel PIR1
             btfss   PIR1,RCIF           ; verifica se ainda tem dado
             goto    RCV#v(_numq)
             movf    RCREG,w             ; obtem o dado transmitido
             
;_numq        set     _numq + 1
;RCV#v(_numq) movf    RCREG,w             ; obtem o dado transmitido
;             btfsc   PIR1,RCIF           ; verifica se ainda tem dado
;             goto    RCV#v(_numq)
 

             ENDM
















; ------------------------------------------------------------------------------
; MACRO: USART_TX                                         Spinelli - 15/02/2011
;
; Transmite o caracter do registrador W para o receptor via porta serial.
; Suporta familias 16F e 18F.
;
; Antes de usar esta macro, a USART deve ser configurada com a macro
; USART_CONFIG
;
; ------------------------------------------------------------------------------
USART_TX     MACRO
_numq      SET     _numq + 1

      IF __FAMILIA == 16
             movwf  TXREG                 ; transmite o caracter para o receptor
             bsf    STATUS,RP0            ;
             btfss  TXSTA,TRMT            ; espera ate que a transmissao termine
             goto   $-1                   ;
             bcf    STATUS,RP0            ;
      ENDIF


      IF __FAMILIA == 18
             movwf  TXREG                 ; transmite o caracter para o receptor
tx1#v(_numq) btfss  TXSTA,TRMT            ; espera ate que a transmissao termine
             goto   tx1#v(_numq)          ;
      ENDIF

             ENDM








; -------------------------------------------------------------------------------------
; XMIT                                                            Spinelli - 03/02/2009
;
; Transmite a string "str" para o dispositivo que estiver lendo a porta serial.
; Antes de usar esta macro, aporta serial deve ser configurada com a macro
; TX_config.
;
;
; Exemplo familia 16F:
;             call      msg002
;
; msg002      XMIT      ms002
; ms002       addwf     PCL,f
;             dt        "mensagem",0x00
;
;
; Exemplo familia 18F:
;             call      msg002
;
; msg002      XMIT      ms002
; ms002       DB        "mensagem",0,0      ; numero par de caracteres
;
; Esta macro destroi o registrador R10.
; -------------------------------------------------------------------------------------
XMIT         MACRO    str
_numq        set      _numq + 1


   IF __FAMILIA<18
             clrf     R10                   ; limpa auxiliar
S4#v(_numq)  movf     R10,w                 ; carrega pointer
             call     str                   ; obtem um caracter da string
             iorlw    0x00                  ; opera com a mascara
             btfsc    STATUS,Z              ; se caracter atual = zero binario
             return                         ; entao achou o fim da string, senao...
             call     XMIT1A
             goto     S4#v(_numq)

       IFNDEF  XMIT_INDICACOR
             variable  XMIT_INDICACOR
XMIT1A       movwf    TXREG                 ; transmite o caracter para o receptor
             bsf      STATUS,RP0            ; banco 1]
S6#v(_numq)  btfss    TXSTA,TRMT            ; espera ate que a transmissao termine
             goto     S6#v(_numq)                  ;
             bcf      STATUS,RP0            ; banco 0
             incf     R10,f                 ; proximo caracter
             return
        ENDIF
   ENDIF



   IF __FAMILIA==18
             movlw   UPPER(str)
             movwf   TBLPTRU
             movlw   HIGH(str)
             movwf   TBLPTRH
             movlw   LOW(str)
             movwf   TBLPTRL
             call    XMITseq0

       ifndef XMACS
             return
       else
             XRETURN
       endif

       IFNDEF  XMIT_INDICACOR
             variable  XMIT_INDICACOR
XMITseq0     tblrd   *+
             movf    TABLAT,w
             xorlw   0x00
             btfsc   STATUS,Z
             return
             movwf   TXREG
XMITseq1     btfss   TXSTA,TRMT
             goto    XMITseq1
             goto    XMITseq0
        ENDIF
   ENDIF
             ENDM


















