; -----------------------------------------------------------------------------
; MODO DE ACERTO DO RELOGIO
;
; Entra neste "modo de acerto" somente quando estiver no "modo normal" e apos o 
; botao de "modo" (RB0) ter sido pressionado e liberado.
;
; Para acertar as horas, pressione e libere o botao de "acerto de horas" 
; em RA4 repetidamente. O RTC sera atualizado (gravado) cada vez que o botao
; for pressionado.
;
; Para acertar os minutos, pressione e libere o botao de "acerto de minutos"
; em RA5 repetidamente. O RTC sera atualizado (gravado) cada vez que o botao
; for pressionado.
;
; Para voltar ao "modo normal" deve-se pressionar e liberar o botao de "modo".
;
; Esta torina zera os segundos.
;
; Nao ha como acertar a data.
;
; mesmo quando esta rotina esta com o controle o display continua sendo
; atualizado.
;
; -----------------------------------------------------------------------------
watch_set      btfss   FLAGS,ACMOD               ; esta mesmo em modo de acerto???
               goto    watch_show                ; NAO --> cai fora
               bcf     INTCON,INTE               ; SIM --> desabilita interrupcao do botao de modo
               bcf     INTCON,INTF               ; aniquila interrupcao pendente se houver

               ;
               ; I/O de todos dos botoes (modo, horas e minutos)
               ;
               bsf     STATUS,RP0                ; banco 1   
               bsf     BTMODO_TRIS,BTMODO        ; I/O do botao de modo
               bsf     ACERTO_TRIS,BTHORA        ; I/O do botao de horas
               bsf     ACERTO_TRIS,BTMIN         ; I/O do botao de minutos
               bcf     EXTLED_TRIS,EXTLED        ; I/O do LED indicador de acerto
               bcf     STATUS,RP0                ; banco 0

               bsf     EXTLED_PORT,EXTLED        ; acende o LED indicador de acerto

proc_acerto    movlw   seg                       ; buffer para ler o DS1302
               movwf   FSR                       ; salva no indexador
               bsf     FLAGS,DS1302_KEY          ; reserva o DS1302
               call    DS1302_GET                ; leitura do RTC DS1302
               bcf     FLAGS,DS1302_KEY          ; libera o DS1302

               movf    ACERTO_PORT,w             ; le a porta dos botoes de acerto
               andlw   ACERTO_IOMSK              ; deixa so os bits necessarios
               movwf   rgsetx                    ; salva a leitura
               movlw   .50                       ; tempo 
               call    delay_ms                  ; debounce do botao

               btfss   rgsetx,BTHORA             ; botao de horas esta pressionado???
               goto    proc_hora                 ; SIM --> processa as horas

               btfss   rgsetx,BTMIN              ; botao de minutos esta pressionado???
               goto    proc_min                  ; SIM --> processa os minutos

               movf    BTMODO_PORT,w             ; le a porta do botao de modo
               andlw   MODO_IOMSK                ; deixa so os bits necessarios
               movwf   rgsetx                    ; salva a leitura
               movlw   .50                       ; tempo 
               call    delay_ms                  ; debounce do botao

               btfsc   rgsetx,BTMODO             ; botao de modo esta pressionado???
               goto    proc_acerto               ; NAO --> repete
proc_freemod   movf    BTMODO_PORT,w             ; SIM --> le a porta do botao de modo
               andlw   MODO_IOMSK                ; deixa so os bits necessarios
               movwf   rgsetx                    ; salva a leitura
               movlw   .50                       ; tempo 
               call    delay_ms                  ; debounce do botao
               btfss   rgsetx,BTMODO             ; botao de modo esta pressionado???
               goto    proc_freemod              ; SIM --> espera liberar o botao
               goto    waset_fim                 ; NAO --> fim

               ;
               ; Processamento das horas
               ;
proc_hora      movf    BTHORA_PORT,w             ; le a porta do botao de hora
               movwf   rgsetx                    ; salva a leitura
               movlw   .50                       ; tempo 
               call    delay_ms                  ; debounce do botao
               btfss   rgsetx,BTHORA             ; botao de horas ainda esta pressionado???
               goto    proc_hora                 ; SIM --> espera liberar o botao

               movf    hor,w                     ; pega a hora corrente (BCD)
               movwf   DIGIT1                    ; salva hora corrente
               rrf     DIGIT1,f                  ; empurra msb para a direita
               rrf     DIGIT1,f                  ; empurra msb para a direita
               rrf     DIGIT1,f                  ; empurra msb para a direita
               rrf     DIGIT1,f                  ; empurra msb para a direita
               movlw   0x0F                      ; mascara
               andwf   DIGIT1,f                  ; deixa so o msb nos bits 0,1,2 e 3

               movf    hor,w                     ; pega a hora corrente (BCD)
               movwf   DIGIT2                    ; salva hora corrente
               movlw   0x0F                      ; mascara
               andwf   DIGIT2,f                  ; deixa so o lsb nos bits 0,1,2 e 3 
               clrf    DSIGN                     ; estabelece que o numero e' positivo
               movlw   .2                        ; quantidade de digitos
               call    dec2bin                   ; converte a binario

               incf    REGA0,f                   ; incrementa as horas
               movlw   .24                       ; limite de horas
               xorwf   REGA0,w                   ; verifica limite
               btfss   STATUS,Z                  ; passou da meia noite???
               goto    waset_set                 ; NAO --> waset_sh
               clrf    REGA0                     ; SIM --> ja e' dia
waset_set      call    bin2dec                   ; converte a decimal
               rlf     DIGIT9,f                  ; ajusta o nible mais alto
               rlf     DIGIT9,f                  ; ajusta o nible mais alto
               rlf     DIGIT9,f                  ; ajusta o nible mais alto
               rlf     DIGIT9,f                  ; ajusta o nible mais alto
               movlw   0xF0                      ; mascara de limpeza
               andwf   DIGIT9,w                  ; deixa so o nible mais alto
               iorwf   DIGIT10,w                 ; junta com o nible mais baixo
               movwf   hor                       ; devolve horas ao buffer
               clrf    seg                       ; zera os segundos
               movlw   seg                       ; endereco do buffer
               movwf   FSR                       ; carrega indexador
               bsf     FLAGS,DS1302_KEY          ; reserva o DS1302
               call    DS1302_PUT                ; altera as horas no RTC
               bcf     FLAGS,DS1302_KEY          ; libera o DS1302
               goto    proc_acerto               ; 

               ;
               ; Processamento dos minutos
               ;
proc_min       movf    BTMIN_PORT,w              ; le a porta do botao de minutos
               movwf   rgsetx                    ; salva a leitura
               movlw   .50                       ; tempo 
               call    delay_ms                  ; debounce do botao
               btfss   rgsetx,BTMIN              ; botao de minutos ainda esta pressionado???
               goto    proc_min                  ; SIM --> espera liberar o botao

               movf    min,w                     ; pega minuto corrente (BCD)
               movwf   DIGIT1                    ; salva minuto corrente
               rrf     DIGIT1,f                  ; empurra msb para a direita
               rrf     DIGIT1,f                  ; empurra msb para a direita
               rrf     DIGIT1,f                  ; empurra msb para a direita
               rrf     DIGIT1,f                  ; empurra msb para a direita
               movlw   0x0F                      ; mascara
               andwf   DIGIT1,f                  ; deixa so o msb nos bits 0,1,2 e 3

               movf    min,w                     ; pega minuto corrente (BCD)
               movwf   DIGIT2                    ; salva minuto corrente
               movlw   0x0F                      ; mascara
               andwf   DIGIT2,f                  ; deixa so o lsb nos bits 0,1,2 e 3 
               clrf    DSIGN                     ; estabelece que o numero e' positivo
               movlw   .2                        ; quantidade de digitos
               call    dec2bin                   ; converte a binario

               incf    REGA0,f                   ; incrementa minuto
               movlw   .60                       ; limite de minuto
               xorwf   REGA0,w                   ; verifica limite
               btfss   STATUS,Z                  ; passou da meia noite???
               goto    waset_setm                ; NAO --> 
               clrf    REGA0                     ; SIM --> 

waset_setm     call    bin2dec                   ; converte minutos a binario
               rlf     DIGIT9,f                  ; ajusta o nible mais alto
               rlf     DIGIT9,f                  ; ajusta o nible mais alto
               rlf     DIGIT9,f                  ; ajusta o nible mais alto
               rlf     DIGIT9,f                  ; ajusta o nible mais alto
               movlw   0xF0                      ; mascara de limpeza
               andwf   DIGIT9,w                  ; deixa so o nible mais alto
               iorwf   DIGIT10,w                 ; junta com o nible mais baixo
               movwf   min                       ; devolve minutos ao buffer
               clrf    seg                       ; zera os segundos
               movlw   seg                       ; endereco do buffer
               movwf   FSR                       ; carrega indexador
               bsf     FLAGS,DS1302_KEY          ; reserva o DS1302
               call    DS1302_PUT                ; altera as minutos no RTC
               bcf     FLAGS,DS1302_KEY          ; libera o DS1302
               goto    proc_acerto               ; 

waset_fim      bcf     EXTLED_PORT,EXTLED        ; apaga o LED indicador de acerto
               bcf     FLAGS,ACMOD               ; limpa flags de acerto ativo
               bcf     INTCON,INTF               ; aniquila interrupcao pendente
               bsf     INTCON,INTE               ; reabilita interrupcao externa
               goto    watch_show                ; vai para modo normal
