; ---------------------------------------------------------------------------------------------------
; 09/08/2014 - Arquivo r3330.asm
;
; RELOGIO COM DISPLAY 3330-5103 OPERADO DE MODO PARALELO DIRETAMENTE PELO PIC16F73 E BASE DE TEMPO 
; COM REAL TIME CLOCK (RTC) DS1302.
;
; ATENCAO: Ao fim da programacao remova o cabo ICSP antes de testar o circuito. 
;
;
; STATUS: FUNCIONA
;
; Esta versao usa botoes para acertar as horas do RTC DS1302.
;
; Somente as horas e minutos serao usados no projeto. A data nao sera usada.
;
; Este relogio de parede usa um display de 7 segmentos 3330-5103 (display de balanca) de 4 digitos
; e um Relogio de Tempo Real (RTC) DS1302 para mostrar as horas.
; ---------------------------------------------------------------------------------------------------




; ---------------------------------------------------------------------------------------------------
; MODOS DE OPERACAO 
; ---------------------------------------------------------------------------------------------------
; Sao previstos 3 modos de operacao de acordo com o estado do pino JP1_PORT<JP1> e do botao 
; BTMODO_PORT<BTMODO> do microcontrolador:
;
; MODO NORMAL ==> Pino JP1_PORT<JP1> = 0  e  botao BTMODO_PORT<BTMODO> liberado
; -----------------------------------------------------------------------------
; No estado normal, o display mostrara as horas e os minutos simplesmente.
; Nesse estado, sera possivel iniciar o acerto dasd horas e minutos pressionando-se
; uma vez o botao de "modo" BTMODO_PORT<BTMODO>.
;
; MODO ACERTO ==> Pino JP1_PORT<JP1> = 0  e  botao BTMODO_PORT<BTMODO> pressionado
; --------------------------------------------------------------------------------
; Ao pressionar e liberar o botao de "modo" BTMODO_PORT<BTMODO>, o sistema entrara
; em "modo de acerto". Nesse modo e1 possivel acertar as horas e minutos pressionando 
; e liberando os botoes BTHORA_PORT<BTHORA> e BTMIN_PORT<BTMIN>. Para voltar ao 
; "modo normal", deve-se pressionar e liberar o botao BTMODO_PORT<BTMODO>, uma unica vez.
;
;
; MODO DEBUG ==> Pino JP1_PORT<JP1> = 1
; -------------------------------------
; Com o jump JP1 ausente, o sistema entrara em "modo debug".
; Nesse modo o display nao sera gerenciado e uma sessao de Hiperterminal de 9600BPS f
; fara conexao com o microcontrolador a fim de verificar o estado do RTC DS1302. 
; Os seguintes comandos sao aceitos:
; O comando "tssmmhhddmmwwaa00" e' usado para acertar data e hora do RTC.
; O comando "r" ou simplesmente um ENTER exibe data e hora no terminal.
; O comando "k" apaga a tela do terminal.
; O comando "z" retorna ao modo normal (relogio) quando estiver no modo debug.
; ---------------------------------------------------------------------------------------------------




; ---------------------------------------------------------------------------------------------------
; GERENCIAMENTO DO DISPLAY 3330-5103
; ---------------------------------------------------------------------------------------------------
; O display nao possui segmento "f" em seu digito de mais alta ordem. 
; A multiplexacao controlada por uma interrupcao de TMR1. O estouro de TMR1 devera ocorrer a cada 
; 4ms aproximadamente e o valor inicial de contagem devera ser 0 (zero) fornecido no simbolo "mplxtimer". 
; Tempos maiores do que 4ms podem fazer o display piscar. Esse tempo foi obtido para um Fosc de 20MHZ.
;
; O display 3330-5103, retirado de uma balanca, possui quatro displays de 7 segmentos, A1, A2, A3
; e A4, onde A4 e' o display do digito de mais baixa ordem. O 3330-5103 e' um display de catodo 
; comum, possui um ponto decimal entre o display A3 e A4 o display A1 nao possui o segmento "f".
;
; Os pinos RA0, RA1, RA2 e RA3 do microcontrolador sao usados para fazer a selecao dos displays
; A1, A2, A3 e A4 respectivamente. Sendo o 3330-5103 um display de catodos comuns, a selecao devera 
; ser feita levando-se um pino RAx ao nivel alto (+5v). Foram usados transistores PNP (BC327) para
; energizar os catodos, assim, os pinos RA0, RA1, RA2 e RA3 do microcontrolador foram conectados com
; resistores de 460 Ohms nas bases dos transistores. O sinal para selecao de  um display deve ser 
; invertido (0v) pois os transistores sao PNP. Ver esquema eletrico abaixo.
;
; Os pinos RB1, RB2, RB3, RB4, RB5, RB6 e RB7 do microcontrolador sao usados para enviar o layout do 
; caracter ao display e sao conectados diretamente (sem resistores) aos pinos 10, 11, 8, 4, 9, 12, e 1
; do display respectivamente.
;
; O pino 5 (dp) do display nao sera conectado pois pertence ao ponto decimal que nao sera usado 
; neste projeto. 
;
; Os layouts dos caracteres sao extraidos da table read "charset" e, como o displays sao de catodos 
; comuns, esse layout sera devidamente invertido pela instrucao "comf" na rotina "send".
;
; ---------------------------------------------------------------------------------------------------



; ---------------------------------------------------------------------------------------------------
; HARDWARE - Ver esquema eletrico abaixo
; ---------------------------------------------------------------------------------------------------
;
; Fosc ............................ Interno - 20MHZ obrigatorio
;                                   A temporizacao da multiplexacao se baseia no clock de 20MHZ.
;                                   Se outra frequencia de clock for usada, o valor do simbolo
;                                   "mplxtimer" devera ser recalculado. 
;
; RA0 ............................. ao pino  7 do display
; RA1 ............................. ao pino  6 do display
; RA2 ............................. ao pino  3 do display
; RA3 ............................. ao pino  2 do display
; RA4 ............................. chave tactil pull_up (active low) avanca horas
; RA5 ............................. chave tactil pull_up (active low) avanca minutos
; RB0 ............................. chave tactil pull_up (active low) entra modo de acerto do relogio
; RB1 ............................. ao pino 10 do display
; RB2 ............................. ao pino 11 do display
; RB3 ............................. ao pino  8 do display
; RB4 ............................. ao pino  4 do display
; RB5 ............................. ao pino  9 do display
; RB6 ............................. ao pino 12 do display
; RB7 ............................. ao pino  1 do display
; RC0 ............................. ao pino CE  do RTC DS1302
; RC1 ............................. ao pino IO  do RTC DS1302
; RC2 ............................. ao pino CLK do RTC DS1302
; RC3 ............................. jump JP1 --> +5v =====>  com resistor 10K entra em modo debug no Hiperterminal
;                                            --> Ground ==>  entra em modo normal do relogio
; RC4 ............................. LED indicador do "modo de acerto"
; RC6 ............................. USART (Tx) - para conexao ao Hiperterminal (9600BPS)
; RC7 ............................. USART (Rx) - para conexao ao Hiperterminal (9600BPS)
;
; ----------------------------------------------------------------------------------------------------------------------------------------------------------------
; Conexoes do display 3330-5103 ao PIC16F73
;  
;
;              RA0  RA1  RA2  RA3                                                                   RB7  RB6  RB5  RB4  RB3  RB2  RB1
;               o    o    o    o                                                                     o    o    o    o    o    o    o
;               |    |    |    |                                                                     |    |    |    |    |    |    |
;               |    |    |    |                                                                     |    |    |    |    |    |    |
;               |    |    |    |                                                                     |    |    |    |    |    |    | 
;               |    |    |    |                                                                     |    |    |    |    |    |    | 
;              +++  +++  +++  +++                                                                    v    v    v    v    v    v    v 
;              | |  | |  | |  | |                                                                        
;              | |  | |  | |  | |                                                       display pin  1   12    9    4    8   11   10
;              | |  | |  | |  | |   DRIVER DE SELECAO DOS DISPLAYS                      ao segmento  a    b    c    d    e    f    g
;              | |  | |  | |  | |   Usar transistores PNP todos os emissores ao +5v                  
;              +++  +++  +++  +++   Todos os resistores sao de 460R.                                     
;               |    |    |    |
;               |    |    |    |
;               |    |    |    |     e 
;               |    |    |    o---b   c---o  ..................................... Milhar
;               |    |    |                |
;               |    |    |     e          |
;               |    |    o---b   c--------------o  ............................... Centena
;               |    |                     |     |
;               |    |     e               |     |
;               |    o---b   c------------------------------------o  .............. Dezena
;               |                          |     |                |
;               |     +5v                  |     |                |
;               |      |                   |     |                |
;               |      |                   |     |                |
;               |      e                   |     |                |
;               o----b   c----------------------------------------------o  ........ Unidade
;                     PNP                  |     |                |     |
;                                          |     |                |     |
;                                          |     |                |     |
;                                          |     |                |     |
;                                          |     |                |     |
;                                   RB6    |     |    RB3  nc     |     |    RB2   RB4   RB0   RB1   RB5
;                                    |     |     |     |    |     |     |     |     |     |     |     |
;                                    |     |     |     |    |     |     |     |     |     |     |     |
;               PINAGEM DO        +--o--+--o--+--o--+--o-+--o--+--o--+--o--+--o--+--o--+--o--+--o--+--o--+
;           DISPLAY 3330-5103     +  1  +  2  +  3  +  4 +  5  +  6  +  7  +  8  +  9  + 10  + 11  + 12  +
;                                 +-----+-----+-----+----+-----+-----+-----+-----+-----+-----+-----+-----+
;           SEGMENTOS ---------->    a                 d    dp                e     c     g     f     b
;     
;                                               a             a             a             a 
;                                            -------       -------       -------       -------      
;                                                   |     |       |     |       |     |       |           
;                                                   |b   f|       |b   f|       |b   f|       |b
;                                               g   |     |   g   |     |   g   |     |   g   |
;                                            -------       -------       -------       ------- 
;                                           |       |     |       |     |       |     |       |
;                                          e|       |c   e|       |c   e|       |c   e|       |c
;                                           |       |     |       |     |       |     |       |
;                                            -------       -------       -------       ------- 
;                                               d             d             d             d 
;                                              A1             A2            A3            A4
;
; NOTAS: 
;  1 - O display A1 nao possui segmengo F.
;  2 - Um capacitor de 1uF deve ser colocado entre o +5v e o GROUND.
;
; ----------------------------------------------------------------------------------------------------------------------------------------------------------------
;
         LIST P=16F73
         #include "P16F73.INC" 
         errorlevel  -302
;

         __CONFIG  _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _HS_OSC
;
          variable _numq
_numq     set       1
;
          #include "macros.inc"
;
          FAMILIAS
;
          #include "simbolos.inc"                ; simbolos usados no programa
          #include "RAM.inc"                     ; memoria RAM
;
; ---------------------------------------------------------------------------------------------
pwron     CODE     0x0000
          goto     inicio
intr      CODE     0x0004
          goto     FLIH                          ; gerenciador de interrupcoes em primeiro nivel
; ---------------------------------------------------------------------------------------------
MAIN      CODE
          HEX2ASCII
          #include "mensagens.inc"               ; mensagens ao Hyperterminal
          #include "charset.inc"                 ; tabela de caracteres para o display

; ---------------------------------------------------------------------------------------------
inicio    movlw   0                              ; 
          movwf   CCP1CON                        ; desliga comparador 1
          movwf   CCP2CON                        ; desliga comparador 2
          clrf    PORTA                          ; limpa os latches da porta de I/O
          clrf    PORTB                          ; limpa os latches da porta de I/O
          clrf    PORTC                          ; limpa os latches da porta de I/O
          bsf     STATUS,RP0                     ; banco 1
          movlw   0x06                           ; todos os bits de PORTA...
          movwf   ADCON1                         ; ...devem ser digitais
          clrf    TRISA                          ; configura I/O 
          clrf    TRISB                          ; configura I/O
          clrf    TRISC                          ; configura I/O
          bsf     JP1_TRIS,JP1                   ; INPUT  --> JP1 jump 
          bsf     BTMODO_TRIS,BTMODO               ; INPUT  --> modo de acerto do relogio
          bsf     BTHORA_TRIS,BTHORA             ; INPUT  --> avanco das horas
          bsf     BTMIN_TRIS,BTMIN               ; INPUT  --> avanco dos minutos
          bcf     STATUS,RP0                     ; banco 0

          movlw   0x0F                           ; mascara
          movwf   DISPLAY                        ; deseleciona todos os displays
          bcf     DISPLAY,CS0                    ; seleciona display das unidades CS0

          clrf    FLAGS                          ; limpa os FLAGS do programa
          call    TMR1_config                    ; configura TMR1 
          call    EXT_config                     ; configura modo de acerto 

          movlw   .255                           ; tempo
          call    delay_ms                       ; espera tudo estabilizar
          bcf     EXTLED_PORT,EXTLED             ; apaga LED indicador modo


; --------------------------------------------------------------------------
; ROTINA PRINCIPAL
;
; O controle sera enviado para um dos tres estados "normal", "debug" ou 
; "acerto", conforme o estado do jump JP1.
;
; O estado "acerto" sera conseguido pressionando e liberando uma unica vez
; o botao BTMODO_PORT<BTMODO>.
;
; Se o jump JP1 esta presente, fazendo com que JP1_PORT<JP1> esteja
; em nivel baixo, o controle sera passado para o "modo normal" de display
; das horas.
;
; Durante o "modo normal", se o botao BTMODO_PORT<BTMODO> for pressionado e
; liberado uma unica vez, o sistema entrara em "modo de acerto".
;
; Se o jump JP1 esta ausente, fazendo com que JP1_PORT<JP1> esteja
; em nivel alto, o controle sera passado para o "modo debug".
;
; --------------------------------------------------------------------------
          movf     JP1_PORT,w                    ; leio o estado do jump JP1
          movwf    R1                            ; salvo o a leitura
          btfss    R1,JP1                        ; jump JP1 ausente???
          goto     modo_normal                   ; SIM --> modo normal
;
modo_debug
          #include "modo_debug.inc"              ; modo para debug do programa

modo_normal 
          #include "modo_normal.inc"             ; modo normal do relogio

modo_acerto
          #include "modo_acerto.inc"             ; modo de acerto do relogio

          goto     modo_normal                   ; isso nao e' necessario, mas...

; --------------------------------------------------------------------------
; Subrotinas especificas para este programa
; --------------------------------------------------------------------------
          #include "interrupts.inc"              ; ISR - tratamento de interrupcoes
          #include "comum.inc"                   ; subrotinas comuns
; --------------------------------------------------------------------------
; Subrotinas genericas
; --------------------------------------------------------------------------
          #include "hexag.asm"
          #include "delay_us.inc"
          #include "delay_ms.inc"
          #include "ds130216f.inc"



         END